@@ -1,116 +1,3 @@
-madogram <- function(data, coord, bins, gev.param = c(0, 1, 0),
-                     which = c("mado", "ext"), xlab, ylab,
-                     angles = NULL, ...){
-  
-  if (nrow(coord) != ncol(data))
-    stop("'data' and 'coord' don't match")
-  
-  n.site <- ncol(data)
-  dist <- distance(coord)
-
-  if (!is.null(angles)){
-    distVec <- distance(coord, vec = TRUE)
-    n.angles <- length(angles)
-    angles.coord <- atan2(distVec[,2], distVec[,1])
-
-    col <- rep(NA, n.site * (n.site - 1) / 2)
-    idx.angles <- list()
-    for (i in 2:n.angles){
-      idx <- which((angles.coord < angles[i]) & (angles.coord >= angles[i-1]))
-      idx.angles <- c(idx.angles, list(idx))
-      col[idx] <- i-1
-    }
-  }
-
-  else
-    col <- 1
- 
-  for (i in 1:n.site){
-    param <- gevmle(data[,i])
-    data[,i] <- gev2frech(data[,i], param[1], param[2],
-                          param[3])
-    data[,i] <- frech2gev(data[,i], gev.param[1],
-                          gev.param[2], gev.param[3])
-  }
-
-  if (missing(bins)){
-    k <- 1
-    mado <- rep(NA, length = n.site * (n.site-1)/2)
-    for (i in 1:(n.site-1)){
-      for (j in (i+1):n.site){
-        mado[k] <- mean(abs(data[,i] - data[,j])) / 2
-        k <- k + 1
-      }
-    }
-  }
-
-  else{
-    bins <- unique(c(0, bins, Inf))  
-    mado <- rep(0, length = length(bins)-1)
-      
-    for (k in 1:(length(bins)-1)){
-      idx <- which((dist <= bins[k+1]) & (dist > bins[k]))
-      
-      if (length(idx)>0){
-        site1 <- (idx-1) %/% n.site + 1
-        site2 <- (idx-1) %% n.site + 1
-        
-        for (i in 1:length(idx))
-          mado[k] <- mado[k] + sum(abs(data[,site1[i]] -
-                                       data[,site2[i]]))
-        
-        mado[k] <- mado[k] / 2 / length(idx) / nrow(data)
-      }
-
-      else
-        mado[k] <- NA
-    }
-  }
-
-  if (gev.param[3] == 0)
-    ext.coeff <- exp(mado/gev.param[2])
-
-  else
-    ext.coeff <- gev2frech(gev.param[1] + mado / gamma(1 - gev.param[3]),
-                           gev.param[1], gev.param[2], gev.param[3])
-
-  if (length(which) == 2){
-    op <- par(mfrow=c(1,2))
-    on.exit(par(op))
-  }
-
-  if (missing(xlab))
-    xlab <- "h"
-
-  if (missing(ylab))
-      ylab <- c(expression(eta(h)), expression(theta(h)))
-    
-  if (any(which == "mado")){
-    if (missing(bins))
-      plot(dist, mado, xlab = xlab, ylab = ylab[1], col = col,
-           pch = col, ...)
-
-    else
-      plot(bins[-1], mado, xlab = xlab, ylab = ylab[1], col = col,
-           pch = col, ...)
-  }
-  
-  if (any(which == "ext")){
-    if (missing(bins))
-      plot(dist, ext.coeff, xlab = xlab, ylab = ylab[2], col = col,
-           pch = col, ...)
-
-    else
-      plot(bins[-1], ext.coeff, xlab = xlab, ylab = ylab[2], col = col,
-           pch = col, ...)
-  }
-
-  if (missing(bins))
-    bins <- c(0, dist)
-  
-  invisible(cbind(bins = bins[-1], madogram = mado, ext.coeff = ext.coeff))
-}
-
 fitextcoeff <- function(data, coord, ..., estim = "ST", marge = "emp",
                         prob = 0, plot = TRUE, loess = TRUE, method = "BFGS",
                         std.err = TRUE, xlab, ylab, angles = NULL,
@@ -161,13 +48,7 @@ fitextcoeff <- function(data, coord, ..., estim = "ST", marge = "emp",
   }
   
   if (marge == "emp") {
-    frech <- data
-    probs <- ppoints(n.obs, a = 0)
-    for (i in 1:n.site){
-      idx <- order(frech[,i])
-      frech[idx,i] <- probs
-    }
-    
+    frech <- apply(data, 2, rank) / (n.obs  +1)    
     frech <- - 1 / log(frech)
     
   }
@@ -249,12 +130,7 @@ fitextcoeff <- function(data, coord, ..., estim = "ST", marge = "emp",
       if (marge == "emp"){
         probs <- ppoints(n.obs - 1, a = 0)
         for (year in 1:n.obs){
-          frech.jack <- data[-year,]
-          for (i in 1:n.site){
-            idx <- order(data[-year,i])
-            frech.jack[idx,i] <- probs
-          }
-          
+          frech.jack <- apply(data[-year,], 2, rank) / n.obs
           frech.jack <- - log(frech.jack)
           
           ext.coeff.std.err[year,] <- .C("extCoeffSmith", as.double(frech.jack),
