@@ -18,8 +18,8 @@ schlatherindfull <- function(data, coord, start, cov.mod = "whitmat", ...,
 
   dist <- distance(coord)
   
-  if (!(cov.mod %in% c("whitmat","cauchy","powexp")))
-    stop("''cov.mod'' must be one of 'whitmat', 'cauchy', 'powexp'")
+  if (!(cov.mod %in% c("whitmat","cauchy","powexp","bessel")))
+    stop("''cov.mod'' must be one of 'whitmat', 'cauchy', 'powexp', 'bessel'")
 
   if (cov.mod == "whitmat")
     cov.mod.num <- 1
@@ -27,6 +27,8 @@ schlatherindfull <- function(data, coord, start, cov.mod = "whitmat", ...,
     cov.mod.num <- 2
   if (cov.mod == "powexp")
     cov.mod.num <- 3
+  if (cov.mod == "bessel")
+    cov.mod.num <- 4
   
   ##First create a "void" function
   nplk <- function(x) x
@@ -40,7 +42,7 @@ schlatherindfull <- function(data, coord, start, cov.mod = "whitmat", ...,
     
     param <- c("alpha", "sill", "range", "smooth", loc.names, scale.names, shape.names)
 
-    body(nplk) <- parse(text = paste("-.C('schlatherindfull', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs),",
+    body(nplk) <- parse(text = paste("-.C('schlatherindfull', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.integer(dist.dim),",
                             paste("as.double(c(", paste(loc.names, collapse = ","), ")), "),
                             paste("as.double(c(", paste(scale.names, collapse = ","), ")), "),
                             paste("as.double(c(", paste(shape.names, collapse = ","), ")), "),
@@ -48,7 +50,7 @@ schlatherindfull <- function(data, coord, start, cov.mod = "whitmat", ...,
   }
 
   else{
-    body(nplk) <- parse(text = paste("-.C('schlatherindfull', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs),",
+    body(nplk) <- parse(text = paste("-.C('schlatherindfull', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.integer(dist.dim),",
                             paste("as.double(rep(1,", n.site, ")), "),
                             paste("as.double(rep(1,", n.site, ")), "),
                             paste("as.double(rep(1,", n.site, ")), "),
@@ -65,6 +67,14 @@ Standard errors are not available unless you fix it.")
     
     std.err.type <- "none"
   }
+
+  if ((cov.mod == "bessel") && !("smooth" %in% names(fixed.param)) && (std.err.type != "none")){
+    if (warn)
+      warning("The Bessel covariance function is not differentiable w.r.t. the ''smooth'' parameter
+Standard errors are not available unless you fix it.")
+    
+    std.err.type <- "none"
+  }
   
   if (std.err.type == "none")
     hessian <- FALSE
@@ -127,14 +137,6 @@ Standard errors are not available unless you fix it.")
   if(any(is.na(m))) 
     stop("'start' specifies unknown arguments")
 
-  ##We use the parscale option to help the optimizer
-  ##We do not overwrite user config
-  if (is.null(control$parscale)){
-    parscale <- abs(unlist(start))
-    parscale[parscale == 0] <- 1
-    control$parscale <- parscale
-  }
-  
   formals(nplk) <- c(f[m], f[-m])
   nllh <- function(p, ...) nplk(p, ...)
 
@@ -307,12 +309,13 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
   ##coord is a matrix giving the coordinates (1 row = 1 station)
   n.site <- ncol(data)
   n.obs <- nrow(data)
+  dist.dim <- ncol(coord)
   n.pair <- n.site * (n.site - 1) / 2
 
   dist <- distance(coord)
      
-  if (!(cov.mod %in% c("whitmat","cauchy","powexp")))
-    stop("''cov.mod'' must be one of 'whitmat', 'cauchy', 'powexp'")
+  if (!(cov.mod %in% c("whitmat","cauchy","powexp","bessel")))
+    stop("''cov.mod'' must be one of 'whitmat', 'cauchy', 'powexp', 'bessel'")
 
   if (cov.mod == "whitmat")
     cov.mod.num <- 1
@@ -320,6 +323,8 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
     cov.mod.num <- 2
   if (cov.mod == "powexp")
     cov.mod.num <- 3
+  if (cov.mod == "bessel")
+    cov.mod.num <- 4
 
   ##With our notation, formula must be of the form y ~ xxxx
   loc.form <- update(loc.form, y ~ .)
@@ -364,24 +369,9 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
   n.pparscale <- scale.model$n.ppar
   n.pparshape <- shape.model$n.ppar
   
-
-  if (n.loccoeff == 1)
-    loc.names <- "locCoeff"
-
-  else
-    loc.names <- paste("locCoeff", 1:n.loccoeff, sep="")
-
-  if (n.scalecoeff == 1)
-    scale.names <- "scaleCoeff"
-
-  else
-    scale.names <- paste("scaleCoeff", 1:n.scalecoeff, sep="")
-
-  if (n.shapecoeff == 1)
-    shape.names <- "shapeCoeff"
-
-  else
-    shape.names <- paste("shapeCoeff", 1:n.shapecoeff, sep="")
+  loc.names <- paste("locCoeff", 1:n.loccoeff, sep="")
+  scale.names <- paste("scaleCoeff", 1:n.scalecoeff, sep="")
+  shape.names <- paste("shapeCoeff", 1:n.shapecoeff, sep="")
   
   param <- c("alpha", "sill", "range", "smooth", loc.names, scale.names, shape.names)
 
@@ -390,7 +380,7 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
 
   ##And define the "body" of the function as the number of parameters
   ##to estimate depends on n.site
-   body(nplk) <- parse(text = paste("-.C('schlatherinddsgnmat', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.double(loc.dsgn.mat), as.double(loc.pen.mat), as.integer(n.loccoeff), as.integer(n.pparloc), as.double(loc.penalty), as.double(scale.dsgn.mat), as.double(scale.pen.mat), as.integer(n.scalecoeff), as.integer(n.pparscale), as.double(scale.penalty), as.double(shape.dsgn.mat), as.double(shape.pen.mat), as.integer(n.shapecoeff), as.integer(n.pparshape), as.double(shape.penalty),",
+   body(nplk) <- parse(text = paste("-.C('schlatherinddsgnmat', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.integer(dist.dim), as.double(loc.dsgn.mat), as.double(loc.pen.mat), as.integer(n.loccoeff), as.integer(n.pparloc), as.double(loc.penalty), as.double(scale.dsgn.mat), as.double(scale.pen.mat), as.integer(n.scalecoeff), as.integer(n.pparscale), as.double(scale.penalty), as.double(shape.dsgn.mat), as.double(shape.pen.mat), as.integer(n.shapecoeff), as.integer(n.pparshape), as.double(shape.penalty),",
                           paste("as.double(c(", paste(loc.names, collapse = ","), ")), "),
                           paste("as.double(c(", paste(scale.names, collapse = ","), ")), "),
                           paste("as.double(c(", paste(shape.names, collapse = ","), ")), "),
@@ -427,14 +417,6 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
   if(any(is.na(m))) 
     stop("'start' specifies unknown arguments")
 
-  ##We use the parscale option to help the optimizer
-  ##We do not overwrite user config
-  if (is.null(control$parscale)){
-    parscale <- abs(unlist(start))
-    parscale[parscale == 0] <- 1
-    control$parscale <- parscale
-  }
-    
   formals(nplk) <- c(f[m], f[-m])
   nllh <- function(p, ...) nplk(p, ...)
 
@@ -451,6 +433,14 @@ Standard errors are not available unless you fix it.")
     
     std.err.type <- "none"
   }
+
+  if ((cov.mod == "bessel") && !("smooth" %in% names(fixed.param)) && (std.err.type != "none")){
+    if (warn)
+      warning("The Bessel covariance function is not differentiable w.r.t. the ''smooth'' parameter
+Standard errors are not available unless you fix it.")
+    
+    std.err.type <- "none"
+  }
   
   if (std.err.type == "none")
     hessian <- FALSE
