@@ -491,21 +491,19 @@ void smithgrad3d(double *data, double *distVec, int *nSite,
 				      frech[k + j * *nObs] * frech[k + j * *nObs]);
 	 
 	  for (l=0;l<*nloccoeff;l++){
-	    dE = scaledsgnmat[i + *nSite * l] * (locs[i] - scales[i] - data[k + i * *nObs]) /
-	      scales[i] / scales[i] / R_pow(frech[k + i * *nObs], shapes[i]) +
-	      scaledsgnmat[j + *nSite * l] * (locs[j] - scales[j] - data[k + j * *nObs]) /
-	      scales[j] / scales[j] / R_pow(frech[k + j * *nObs], shapes[j]);
+	    dE = (shapes[i] - 1) * locdsgnmat[i + *nSite * l] /
+	      scales[i] / R_pow(frech[k + i * *nObs], shapes[i]) +
+	      (shapes[j] - 1) * locdsgnmat[j + *nSite * l] /
+	      scales[j] / R_pow(frech[k + j * *nObs], shapes[j]);
 
-	    dz1scale = - R_pow(frech[k + i * *nObs], 1 - shapes[i]) *
-	      (data[k + i * *nObs] - locs[i]) / scales[i] / scales[i] *
-	      scaledsgnmat[i + *nSite * l];
-	    dz2scale = - R_pow(frech[k + j * *nObs], 1 - shapes[j]) *
-	      (data[k + j * *nObs] - locs[j]) / scales[j] / scales[j] *
-	      scaledsgnmat[j + *nSite * l];
+	    dz1loc = - R_pow(frech[k + i * *nObs], 1 - shapes[i]) /
+	      scales[i] * locdsgnmat[i + *nSite * l];
+	    dz2loc = - R_pow(frech[k + j * *nObs], 1 - shapes[j]) /
+	      scales[j] * locdsgnmat[j + *nSite * l];	    
 
-	    grad[(6 + *nloccoeff + l) * *nObs + k] += (dAz1 * dz1scale + dAz2 * dz2scale) +
-	      ((dBz1 * dz1scale + dBz2 * dz2scale) * C + B * 
-	       (dCz1 * dz1scale + dCz2 * dz2scale) + (dDz1 * dz1scale + dDz2 * dz2scale)) /
+	    grad[(6 + *nloccoeff + l) * *nObs + k] += (dAz1 * dz1loc + dAz2 * dz2loc) +
+	      ((dBz1 * dz1loc + dBz2 * dz2loc) * C + B * 
+	       (dCz1 * dz1loc + dCz2 * dz2loc) + (dDz1 * dz1loc + dDz2 * dz2loc)) /
 	      (B * C + D) + dE;
 	  }
 
@@ -599,6 +597,11 @@ void schlathergrad(int *covmod, double *data, double *dist, int *nSite,
   case 3:
     flag = powerExp(dist, nPairs, *sill, *range, *smooth, rho);
     break;
+  case 4:
+    //Here we use 0 for dim as we don't care for the computation of
+    //the gradient
+    flag = bessel(dist, nPairs, 0, *sill, *range, *smooth, rho);
+    break;
   }
   
   //Compute the GEV parameters using the design matrix
@@ -684,7 +687,15 @@ void schlathergrad(int *covmod, double *data, double *dist, int *nSite,
 	  grad[2 * *nObs + k] -= rho[currentPair] *
 	    R_pow(dist[currentPair] / *range, *smooth) * log(dist[currentPair] / *range) *
 	    jacCommonRho;
-	  break;	   
+	  break;
+	case 4:
+	  //i.e. Bessel
+	  grad[*nObs + k] += rho[currentPair] * dist[currentPair] * 
+	    bessel_j(dist[currentPair] / *range, *smooth + 1) /
+	    (bessel_j(dist[currentPair] / *range, *smooth) * *range * *range) *
+	    jacCommonRho;
+	  grad[2 * *nObs + k] = R_NaReal;
+	  break;
 	}
       }
     }
@@ -849,6 +860,11 @@ void schlatherindgrad(int *covmod, double *data, double *dist, int *nSite,
   case 3:
     flag = powerExp(dist, nPairs, *sill, *range, *smooth, rho);
     break;
+  case 4:
+    //Here we use 0 for dim as we don't care for the computation of
+    //the gradient
+    flag = bessel(dist, nPairs, 0, *sill, *range, *smooth, rho);
+    break;
   }
   
   //Compute the GEV parameters using the design matrix
@@ -948,7 +964,15 @@ void schlatherindgrad(int *covmod, double *data, double *dist, int *nSite,
 	  grad[3 * *nObs + k] -= rho[currentPair] *
 	    R_pow(dist[currentPair] / *range, *smooth) * log(dist[currentPair] / *range) *
 	    jacCommonRho;
-	  break;	   
+	  break;
+	case 4:
+	  //i.e. Bessel
+	  grad[*nObs + k] += rho[currentPair] * dist[currentPair] * 
+	    bessel_j(dist[currentPair] / *range, *smooth + 1) /
+	    (bessel_j(dist[currentPair] / *range, *smooth) * *range * *range) *
+	    jacCommonRho;
+	  grad[2 * *nObs + k] = R_NaReal;
+	  break;
 	}
       }
     }
@@ -1157,7 +1181,7 @@ void geomgaussgrad(int *covmod, double *data, double *dist, int *nSite,
   frech = (double *)R_alloc(*nObs * *nSite, sizeof(double));
   
   //Stage 0: Compute the covariance at each location
-  flag = geomCovariance(dist, nPairs, *covmod, *sigma2, *sill, *range,
+  flag = geomCovariance(dist, nPairs, 0, *covmod, *sigma2, *sill, *range,
 			*smooth, mahalDist);
   
   //Compute the GEV parameters using the design matrix
@@ -1265,7 +1289,15 @@ void geomgaussgrad(int *covmod, double *data, double *dist, int *nSite,
 	  grad[3 * *nObs + k] += *sigma2 * rho *
 	    R_pow(dist[currentPair] / *range, *smooth) * log(dist[currentPair] / *range) /
 	    mahalDist[currentPair] * jacCommon;
-	  break;	   
+	  break;
+	case 4:
+	  //i.e. Bessel
+	  grad[2 * *nObs + k] -= *sigma2 * rho * dist[currentPair] * 
+	    bessel_j(dist[currentPair] / *range, *smooth + 1) /
+	    (bessel_j(dist[currentPair] / *range, *smooth) * *range * *range *
+	     mahalDist[currentPair]) * jacCommon;
+	  grad[3 * *nObs + k] = R_NaReal;
+	  break;
 	}
       }
     }
