@@ -1,6 +1,7 @@
 #include <R.h>
 #include <Rmath.h>
 #include <Rinternals.h>
+#include <R_ext/Lapack.h>
 
 #define MINF -1.0e15
 #define EPS DBL_EPSILON
@@ -9,10 +10,10 @@
 //  From schlather.c
 //
 void schlatherfull(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-		   double *locs, double *scales, double *shapes, double *sill,
+		   int *dim, double *locs, double *scales, double *shapes, double *sill,
 		   double *range, double *smooth, int *fitmarge, double *dns);
 void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-		      double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
+		      int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
 		      double *locpenalty, double *scaledsgnmat, double *scalepenmat,
 		      int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
 		      double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
@@ -23,11 +24,11 @@ void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *
 //  From schlatherind.c
 //
 void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
-		      int *nObs, double *locs, double *scales, double *shapes,
+		      int *nObs, int *dim, double *locs, double *scales, double *shapes,
 		      double *alpha, double *sill, double *range, double *smooth,
 		      int *fitmarge,double *dns);
 void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-			 double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
+			 int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
 			 double *locpenalty, double *scaledsgnmat, double *scalepenmat,
 			 int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
 			 double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
@@ -38,11 +39,11 @@ void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, in
 //  From geomgauss.c
 //
 void geomgaussfull(int *covmod, double *data, double *dist, int *nSite,
-		   int *nObs, double *locs, double *scales, double *shapes,
+		   int *nObs, int *dim, double *locs, double *scales, double *shapes,
 		   double *sigma2, double *sill, double *range, double *smooth,
 		   int *fitmarge,double *dns);
 void geomgaussdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-		      double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
+		      int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
 		      double *locpenalty, double *scaledsgnmat, double *scalepenmat,
 		      int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
 		      double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
@@ -53,12 +54,12 @@ void geomgaussdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *
 //  From nsgeomgauss.c
 //
 void nsgeomgaussfull(int *covmod, double *data, double *dist, int *nSite,
-		     int *nObs, double *locs, double *scales, double *shapes,
+		     int *nObs, int *dim, double *locs, double *scales, double *shapes,
 		     double *sigma2dsgnmat, double *sigma2coeff, int *nsigma2coeff,
 		     double *sill, double *range, double *smooth, int *fitmarge,
 		     double *dns);
 void nsgeomgaussdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-			double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
+			int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
 			double *locpenalty, double *scaledsgnmat, double *scalepenmat,
 			int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
 			double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
@@ -136,15 +137,17 @@ double cauchy(double *dist, int nPairs, double sill, double range,
 	      double smooth, double *rho);
 double powerExp(double *dist, int nPairs, double sill, double range,
 		double smooth, double *rho);
+double bessel(double *dist, int nPairs, int dim, double sill,
+	      double range, double smooth, double *rho);
 double mahalDistFct(double *distVec, int nPairs, double *cov11,
 		    double *cov12, double *cov22, double *mahal);
 double mahalDistFct3d(double *distVec, int nPairs, double *cov11,
 		      double *cov12, double *cov13, double *cov22, 
 		      double *cov23, double *cov33, double *mahal);
-double geomCovariance(double *dist, int nPairs, int covmod,
+double geomCovariance(double *dist, int nPairs, int dim, int covmod,
 		      double sigma2, double sill, double range,
 		      double smooth, double *rho);
-double nsgeomCovariance(double *dist, int nSite, int covmod,
+double nsgeomCovariance(double *dist, int nSite, int dim, int covmod,
 			double *sigma2, double sill, double range,
 			double smooth, double *rho);
 
@@ -230,13 +233,13 @@ void fitcovmat3d(double *cov11, double *cov12, double *cov13,
 		 int *nPairs, double *dist, double *extcoeff,
 		 double *weights, double *ans);
 void fitcovariance(int *covmod, double *sill, double *range, double *smooth,
-		   int *nPairs, double *distVec, double *extcoeff,
+		   int *nPairs, int *dim, double *distVec, double *extcoeff,
 		   double *weights, double *ans);
 void fiticovariance(int *covmod, double *alpha, double *sill, double *range,
-		    double *smooth, int *nPairs, double *dist, double *extcoeff,
+		    double *smooth, int *nPairs, int *dim, double *dist, double *extcoeff,
 		    double *weights, double *ans);
 void fitgcovariance(int *covmod, double *sigma2, double *sill, double *range,
-		    double *smooth, int *nPairs, double *dist, double *extcoeff,
+		    double *smooth, int *nPairs, int *dim, double *dist, double *extcoeff,
 		    double *weights, double *ans);
 
 ///////////////////////////////////
@@ -257,3 +260,50 @@ void spatgevlik(double *data, double *covariables, int *nSite, int *nObs,
 void madogram(double *data, int *nObs, int *nSite, double *mado);
 void lmadogram(double *data, int *nObs, int *nSite, double *lambda,
 	       int *nLambda, double *lmado);
+
+///////////////////////////////////
+//  From simsmith.c
+//
+void rsmith1d(double *coord, double *center, double *edge, int *nObs,
+	      int *nSites, double *var, double *ans);
+void rsmith2d(double *coord, double *center, double *edge, int *nObs,
+	      int *nSites, int *grid, double *cov11, double *cov12,
+	      double *cov22, double *ans);
+
+///////////////////////////////////
+//  From direct.c
+//
+void buildcovmat(int *nSite, int *grid, int *covmod, double *coord, int *dim,
+		 double *nugget, double *sill, double *range, double *smooth,
+		 double *covMat);
+void direct(int *n, int *nSite, int *grid, int *covmod, double *coord, int *dim,
+	    double *nugget, double *sill, double *range, double *smooth,
+	    double *ans);
+
+///////////////////////////////////
+//  From randomlines.c
+//
+void vandercorput(int *n, double *coord);
+void rotation(double *coord, int *n, double *u, double *v, double *w,
+	      double *angle);
+
+///////////////////////////////////
+//  From turningbands.c
+//
+void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid, 
+	 double *coord, double *nugget, double *sill, double *range,
+	 double *smooth, int *nlines, double *ans);
+void tbmcore(int *nsite, int *neffSite, int *dim, int *covmod,
+	     int *grid, double *coord, double *nugget, double *sill,
+	     double *range, double *smooth, int *nlines, double *lines,
+	     double *ans);
+
+///////////////////////////////////
+//  From simschlather.c
+//
+void rschlathertbm(double *coord, int *nObs, int *nSites, int *dim,
+		   int *covmod, int *grid, double *sill, double *range,
+		   double *smooth, double *ans);
+void rschlatherdirect(double *coord, int *nObs, int *nSites, int *dim,
+		      int *covmod, int *grid, double *sill, double *range,
+		      double *smooth, double *ans);
